using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Components.Calendar.DayCal.Class;
using System.Drawing.Drawing2D;
using Components.Calendar.DayCal.GUI;
using Components.Calendar.DayCal.Class.Event;
using System.Drawing.Design;
using Components.TypeEditors;

namespace Components.Calendar.DayCal.GUI
{
    /// <summary>    
    /// Component which represents a day calendar    
    /// </summary>
    public partial class DayCal : UserControl
    {
        #region Variables
        /// <summary>
        /// Data for day displaying
        /// </summary>
        private Hours vHours = new Hours();
        /// <summary>
        /// Position of the autoscroll
        /// </summary>
        private Point vAutoScrollPosition = new Point();
        /// <summary>
        /// Indicates the number of lines a scroll will move
        /// </summary>
        private short vLinesToScroll = 1;
        /// <summary>
        /// Information about the meeting
        /// </summary>        
        private MeetingRow vMeetingRow = new MeetingRow();
        /// <summary>
        /// Indicates whether the component was resized
        /// </summary>
        private bool vIsResized = false;
        /// <summary>
        /// Indicates that the user scrolled
        /// </summary>
        private bool vIsScrolled = true;
        /// <summary>
        /// Indicates the first hour to diplay
        /// </summary>
        private DateTime vFirstHourToDisplay = new DateTime(1, 1, 1, 8, 30, 00);

        #region FocusCal
        /// <summary>
        /// Indicates whether a meeting is creating
        /// </summary>
        private Boolean vIsMeetingCreating = false;
        /// <summary>
        /// Indicates whether the mouse isused to select area of time
        /// </summary>
        private Boolean vHaveMouse;
        /// <summary>
        /// Coordinates of the starting point of the selection 
        /// </summary>
        private Point vStartingFocusePoint = new Point();
        /// <summary>
        /// Coordinates of the ending point of the selection
        /// </summary>
        private Point vPreviousFocusePoint = new Point();
        #endregion
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        public DayCal()
        {
            InitializeComponent();

            #region Autoscroll
            vAutoScrollPosition = this.AutoScrollPosition;
            vHours.SizeOfComponent = pb_Day.Size;
            pb_Day.Height = vHours.Height;
            this.AutoScroll = true;
            #endregion
        }
        #endregion

        #region Methods
        /// <summary>
        /// Fill the component
        /// </summary>
        /// <param name="g">Object Graphic fir desing</param>
        protected void Redraw(Graphics g)
        {
            foreach (DayRectangleInfo ri in vHours.Recalculate())
            {
                //Rectangle filling
                LinearGradientBrush lgb =
                    new LinearGradientBrush(ri.Rectangle, ri.BackgroundColor, ri.BackColor2, LinearGradientMode.Horizontal);
                g.FillRectangle(lgb, ri.Rectangle);

                switch (ri.Position)
                {
                    case MeetingPosition.StartingMeeting:
                        #region StartingMeeting
                        //Upper line
                        g.DrawLine(new Pen(Color.Black), ri.Rectangle.Location,
                            new Point(ri.Rectangle.Location.X + vHours.SizeOfComponent.Width, ri.Rectangle.Location.Y));

                        g.DrawString(ri.Text, ri.TextFont, new SolidBrush(ri.TextColor), ri.Rectangle.Location);
                        break;
                        #endregion
                    case MeetingPosition.Meeting:
                        #region Meeting

                        break;
                        #endregion
                    case MeetingPosition.EndingMeeting:
                        #region EndingMeeting
                        //Downer line
                        g.DrawLine(new Pen(Color.Black), new Point(ri.Rectangle.X, ri.Rectangle.Y + vHours.HeaderHeight),
                            new Point((ri.Rectangle.X - 1) + ri.Rectangle.Width, ri.Rectangle.Y + vHours.HeaderHeight));
                        break;
                        #endregion
                    case MeetingPosition.OutOfMeeting:
                        #region OutOfMeeting
                        g.FillRectangle(new SolidBrush(ri.BackgroundColor), ri.Rectangle);

                        //Upper line
                        g.DrawLine(ri.BorderPen, ri.Rectangle.Location,
                            new Point(ri.Rectangle.Location.X + ri.Rectangle.Width, ri.Rectangle.Location.Y));

                        //Downer line
                        g.DrawLine(ri.BorderPen, new Point(ri.Rectangle.X, ri.Rectangle.Y + vHours.HeaderHeight),
                            new Point((ri.Rectangle.X - 1) + ri.Rectangle.Width, ri.Rectangle.Y + vHours.HeaderHeight));

                        g.DrawString(ri.Text, ri.TextFont, new SolidBrush(ri.TextColor), ri.Rectangle.Location);
                        break;
                        #endregion
                }
            }
        }
        /// <summary>
        /// Scroll up or down the DayCal
        /// </summary>
        /// <param name="_ScrollDirection">The scroll direction</param>
        protected void Wheel(ScrollDirection _ScrollDirection)
        {
            Graphics g = pb_Day.CreateGraphics();

            switch (_ScrollDirection)
            {
                case ScrollDirection.MaximumUp:
                case ScrollDirection.MaximumDown:
                    break;
                case ScrollDirection.Down:
                    g.TranslateClip(0, vLinesToScroll * Hours.HEADER_HEIGHT);
                    break;
                case ScrollDirection.Up:
                    g.TranslateClip(0, -(vLinesToScroll) * Hours.HEADER_HEIGHT);
                    break;
            }
        }
        /// <summary>
        /// Shows the first hour of the day
        /// </summary>
        public void DisplayFirstHour()
        {
            int Minutes = (vFirstHourToDisplay.Hour * 60) + vFirstHourToDisplay.Minute;
            int Line = Minutes / TimeOfMeeting;
            if (Line <= Hours.DAYS_PARTS)
            {
                this.vAutoScrollPosition = new Point(vAutoScrollPosition.X, Line * Hours.HEADER_HEIGHT);
                //this.Refresh();
                OnPaint(new PaintEventArgs(pb_Day.CreateGraphics(), new Rectangle(pb_Day.Location, pb_Day.Size)));
            }
        }
        /// <summary>
        /// Override of the OnPaint which notify the Paint event
        /// </summary>
        /// <param name="e"></param>
        protected override void OnPaint(PaintEventArgs e)
        {
            Graphics g = e.Graphics;//pb_Day.CreateGraphics();

            if (this.vIsResized)
            {
                foreach (DayRectangleInfo ri in vHours.Recalculate())
                {
                    LinearGradientBrush lgb =
                        new LinearGradientBrush(ri.Rectangle, ri.BackgroundColor, ri.BackColor2, LinearGradientMode.Horizontal);
                    g.FillRectangle(lgb, ri.Rectangle);

                    switch (ri.Position)
                    {
                        case MeetingPosition.StartingMeeting:
                            #region StartingMeeting
                            //Ligne suprieure
                            g.DrawLine(new Pen(Color.Black), ri.Rectangle.Location,
                                new Point(ri.Rectangle.Location.X + vHours.SizeOfComponent.Width, ri.Rectangle.Location.Y));

                            g.DrawString(ri.Text, ri.TextFont, new SolidBrush(ri.TextColor), ri.Rectangle.Location);
                            break;
                            #endregion

                        case MeetingPosition.Meeting:
                            #region Meeting

                            break;
                            #endregion

                        case MeetingPosition.EndingMeeting:
                            #region EndingMeeting
                            //Ligne infrieure
                            g.DrawLine(new Pen(Color.Black), new Point(ri.Rectangle.X, ri.Rectangle.Y + vHours.HeaderHeight),
                                new Point((ri.Rectangle.X - 1) + ri.Rectangle.Width, ri.Rectangle.Y + vHours.HeaderHeight));
                            break;
                            #endregion

                        case MeetingPosition.OutOfMeeting:
                            #region OutOfMeeting
                            g.FillRectangle(new SolidBrush(ri.BackgroundColor), ri.Rectangle);

                            //Ligne suprieure
                            g.DrawLine(ri.BorderPen, ri.Rectangle.Location,
                                new Point(ri.Rectangle.Location.X + ri.Rectangle.Width, ri.Rectangle.Location.Y));

                            //Ligne infrieure
                            g.DrawLine(ri.BorderPen, new Point(ri.Rectangle.X, ri.Rectangle.Y + vHours.HeaderHeight),
                                new Point((ri.Rectangle.X - 1) + ri.Rectangle.Width, ri.Rectangle.Y + vHours.HeaderHeight));

                            g.DrawString(ri.Text, ri.TextFont, new SolidBrush(ri.TextColor), ri.Rectangle.Location);
                            break;
                            #endregion
                    }
                }
                this.vIsResized = false;
            }
            if (this.vIsScrolled)
            {
                this.AutoScrollPosition = this.vAutoScrollPosition;
                this.vIsScrolled = false;
            }
            base.OnPaint(e);
        }
        #endregion

        #region Event
        /// <summary>
        /// Click in the DayCal
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Clicks < 2)
            {
                this.Focus();

                if (e.Button == MouseButtons.Left)
                {
                    this.vIsMeetingCreating = true;

                    #region Prparation du rendez-vous
                    foreach (DayRectangleInfo ri in vHours.ListToDraw)
                    {
                        if (ri.Rectangle.Contains(e.Location))
                        {
                            vMeetingRow.StartingDate = vHours.GetDate(ri.Rectangle.Y / vHours.HeaderHeight);
                            vStartingFocusePoint = ri.Rectangle.Location;
                            break;
                        }
                    }
                    #endregion

                    #region StartingDate focus
                    this.vStartingFocusePoint = new Point(vHours.HeaderWidth, e.Y);
                    this.vStartingFocusePoint.Offset(pb_Day.Left, pb_Day.Top);
                    this.vPreviousFocusePoint = vStartingFocusePoint;
                    this.vHaveMouse = true;
                    #endregion
                }
            }
        }
        /// <summary>
        /// The mouse moved
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MouseMove(object sender, MouseEventArgs e)
        {
            if (vHaveMouse)
            {
                Point pCurrent = new Point(vHours.HeaderWidth, e.Y);
                pCurrent.Offset(pb_Day.Left, pb_Day.Top);

                //Prparation sur l'ancien rectangle
                Rectangle OldRec = RectangleToScreen(new Rectangle(vStartingFocusePoint,
                    new Size(pb_Day.Width - vHours.HeaderWidth, vPreviousFocusePoint.Y - vStartingFocusePoint.Y)));

                //Prparation du nouveau rectangle
                Rectangle NewRec = RectangleToScreen(new Rectangle(vStartingFocusePoint,
                    new Size(pb_Day.Width - vHours.HeaderWidth, pCurrent.Y - vStartingFocusePoint.Y)));

                //Dessinage                
                ControlPaint.FillReversibleRectangle(OldRec, Color.Black);
                ControlPaint.FillReversibleRectangle(NewRec, Color.Black);

                vPreviousFocusePoint = pCurrent;
            }
        }
        /// <summary>
        /// Unclick on the DayCal
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MouseUp(object sender, MouseEventArgs e)
        {
            if (vHaveMouse)
            {
                //Identifiant du rendez-vous
                int IDMtg = -1;

                if (e.Button == MouseButtons.Left && this.vIsMeetingCreating)
                {

                    #region Prparation du rendez-vous
                    foreach (DayRectangleInfo ri in vHours.ListToDraw)
                    {
                        if (ri.Rectangle.Contains(e.Location))
                        {
                            this.vMeetingRow.EndingDate = vHours.GetDate(ri.Rectangle.Y / vHours.HeaderHeight);
                            vPreviousFocusePoint = ri.Rectangle.Location;
                            IDMtg = ri.InternalID;//this.vDateToDisplay.ID;
                            break;
                        }
                    }
                    #endregion

                    #region Stop focus
                    vHaveMouse = false;
                    #endregion
                }

                #region Effacage du rectangle
                Point pCurrent = new Point(vHours.HeaderWidth, e.Y);
                pCurrent.Offset(pb_Day.Left, pb_Day.Top);

                //Prparation du nouveau rectangle
                Rectangle NewRec = RectangleToScreen(new Rectangle(vStartingFocusePoint,
                    new Size(pb_Day.Width - vHours.HeaderWidth, pCurrent.Y - vStartingFocusePoint.Y)));

                //Dessinage                            
                ControlPaint.FillReversibleRectangle(NewRec, Color.Black);
                #endregion

                #region Prparation d'un vnement de demande ou de modification de rendez-vous
                if (vIsMeetingCreating)
                {
                    if (this.vMeetingRow.EndingDate < this.vMeetingRow.StartingDate)
                        this.vIsMeetingCreating = false;

                    //Demande de cration de rendez-vous
                    else if (this.vMeetingRow.StartingDate < this.vMeetingRow.EndingDate)
                        OnMeetingCreation(new MeetingEventArgs(this.vMeetingRow));

                    //Vrification si c'est un rdv  supprimer ou  modifier
                    else if (IDMtg != -1)
                    {
                        bool Continue = false;
                        foreach (MeetingRow mr in vHours.Hour)
                        {
                            if (mr.ID == IDMtg)
                            {
                                this.vMeetingRow.StartingDate = mr.StartingDate;
                                this.vMeetingRow.EndingDate = mr.EndingDate;
                                this.vMeetingRow.Title = mr.Title;
                                this.vMeetingRow.MeetingData = mr.MeetingData;
                                this.vMeetingRow.SetID(IDMtg);
                                Continue = true;
                            }
                        }
                        if (Continue)
                            OnMeetingModification(new MeetingEventArgs(this.vMeetingRow));
                    }
                }
                #endregion

                this.vIsMeetingCreating = false;
            }
        }
        /// <summary>
        /// The size of the DayCal changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_SizeChanged(object sender, EventArgs e)
        {
            this.vIsResized = true;
            vHours.SizeOfComponent = pb_Day.Size;
        }
        /// <summary>
        /// Event Paint
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Paint(object sender, PaintEventArgs e) { this.Redraw(e.Graphics); }
        #endregion

        #region Personal Event
        /// <summary>
        /// Event thrown when a meeting creation is asked
        /// </summary>
        [Category("DayCal - Event")]
        [Description("Event thrown when a meeting creation is asked")]
        public event MeetingEventHandler MeetingCreation;
        /// <summary>
        /// Notify meeting creation
        /// </summary>
        protected virtual void OnMeetingCreation(MeetingEventArgs args)
        {
            if (MeetingCreation != null)
                MeetingCreation(this, args);
        }
        /// <summary>
        /// Event thrown when a meeting modification is asked
        /// </summary>
        [Category("DayCal - Event")]
        [Description("Event thrown when a meeting modification is asked")]
        public event MeetingEventHandler MeetingModification;
        /// <summary>
        /// Notify a meeting modification
        /// </summary>
        /// <param name="args"></param>
        protected virtual void OnMeetingModification(MeetingEventArgs args)
        {
            if (MeetingModification != null)
                MeetingModification(this, args);
        }
        /// <summary>
        /// Event thrown when the date to display changed
        /// </summary>
        [Category("DayCal - Event")]
        [Description("Event thrown when a meeting modification is asked")]
        public event DisplayDateEventHandler DisplayDateChanged;
        /// <summary>
        /// Notify a change of date to display
        /// </summary>
        /// <param name="args"></param>
        protected virtual void OnDisplayDateChanged(DisplayDateEventArgs args)
        {
            if (DisplayDateChanged != null)
                DisplayDateChanged(this, args);
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the first hour to display
        /// </summary>
        [Category("DayAgenda-Options")]
        [Description("The first hour to display in the calendar")]
        [Editor(typeof(UIHourEditor), typeof(UITypeEditor))]
        public string FirstHourToDisplay
        {
            get { return this.vFirstHourToDisplay.ToShortTimeString(); }
            set
            {
                DateTime t = DateTime.Parse(value);
                this.vFirstHourToDisplay = t;
                this.vIsScrolled = true;
                DisplayFirstHour();
            }
        }
        /// <summary>
        /// Gets or sets the date to display in the DayCal
        /// </summary>
        [Category("DayAgenda-Options")]
        [Description("The date to display in the DayCal")]
        [Browsable(true)]
        public DateTime DateToDisplay
        {
            get { return this.vHours.DateToDisplay; }
            set
            {
                DateTime Before = this.vHours.DateToDisplay;
                this.vHours.DateToDisplay = value;
                OnDisplayDateChanged(new DisplayDateEventArgs(value, Before));
            }
        }
        /// <summary>
        /// Donne la liste des remarques des rendez-vous
        /// </summary>
        [Browsable(false)]
        public IMeetingRowList MeetingSource
        {
            get
            {
                RootMeetingSet mr = new RootMeetingSet(this.vHours.MeetingList);
                return mr as IMeetingRowList;
            }
            set
            {
                this.vHours.MeetingList = value.MeetingList;
                Redraw(pb_Day.CreateGraphics());
            }
        }
        /// <summary>
        /// Gets the laps of time (in minutes) of a rendez-vous
        /// </summary>
        [Browsable(false)]
        public static int TimeOfMeeting
        {
            get
            {
                float ret = 24;
                ret /= Hours.DAYS_PARTS;
                ret *= 60;
                if (ret == 0) ret = 1;
                return (int)ret;
            }
        }

        #region Colors OptionContainer
        /// <summary>
        /// Retourne ou initialise le couleur de fond des heures sans rendez-vous
        /// </summary>
        [Category("DayAgenda-Options Color")]
        [Description("Background color of a non-meeting")]
        public Color NonMeetingBackColor
        {
            get { return vHours.NonMeetingBackColor; }
            set
            {
                vHours.NonMeetingBackColor = value;
                vHours.Recalculate();
                this.Refresh();
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur de fond d'un jour avec rendez-vous
        /// </summary>
        [Category("DayAgenda-Options Color")]
        [Description("Second Color a the fading color of a meeting")]
        public Color MeetingFadeColorTwo
        {
            get { return vHours.SecondMeetingBackcolor; }
            set
            {
                vHours.SecondMeetingBackcolor = value;
                vHours.Recalculate();
                this.Refresh();
            }
        }
        /// <summary>
        /// Retourne ou initialise la couleur pour le dgrad de couleur d'un rendez-vous
        /// </summary>
        [Category("DayAgenda-Options Color")]
        [Description("First Color a the fading color of a meeting")]
        public Color MeetingFadeColorOne
        {
            get { return vHours.FirstMeetingFadeColor; }
            set
            {
                vHours.FirstMeetingFadeColor = value;
                vHours.Recalculate();
                this.Refresh();
            }

        }
        /// <summary>
        /// Retourne ou initialise la couleur du texte d'une heure avec rendez-vous
        /// </summary>
        [Category("DayAgenda-Options Color")]
        [Description("Color of the text of a meeting")]
        public Color MeetingTextColor
        {
            get { return vHours.RdvTextColor; }
            set
            {
                vHours.RdvTextColor = value;
                vHours.Recalculate();
                Refresh();
            }
        }
        /// <summary>
        /// Retourne ou initialise la police d'criture d'une heure avec rendez-vous
        /// </summary>
        [Category("DayAgenda-Options")]
        [Description("Font of the text of a meeting")]
        public Font MeetingTextFont
        {
            get { return vHours.RdvTextFont; }
            set
            {
                vHours.RdvTextFont = value;
                vHours.Recalculate();
                Refresh();
            }
        }
        /// <summary>
        /// Gets or sets the background color of the header
        /// </summary>
        [Category("DayAgenda-Options Color")]
        [Description("Background color of the header")]
        public Color HeaderBackgroundColor
        {
            get { return this.vHours.HeaderBackgroundColor; }
            set
            {
                this.vHours.HeaderBackgroundColor = value;
                this.Refresh();
            }
        }
        /// <summary>
        /// Gets or sets the color of the header's text
        /// </summary>
        [Category("DayAgenda-Options Color")]
        [Description("Color of the header's text")]
        public Color HeaderTextColor
        {
            get { return this.vHours.HeaderTextColor; }
            set
            {
                this.vHours.HeaderTextColor = value;
                this.Refresh();
            }
        }
        /// <summary>
        /// Gets or sets the font header's text
        /// </summary>
        [Category("DayAgenda-Options")]
        [Description("Font of the header's text")]
        public Font HeaderTextFont
        {
            get { return this.vHours.HeaderTextFont; }
            set
            {
                this.vHours.HeaderTextFont = value;
                this.Refresh();
            }
        }
        #endregion
        #endregion
    }
}
